"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfigState = exports.OrchestratorApi = exports.BaseCrudService = void 0;
const request_1 = __importDefault(require("request"));
const logger_1 = require("./logger");
const utils_1 = require("./utils");
const excel_csv_read_write_1 = require("excel-csv-read-write");
const logger = logger_1.getLogger('main');
/**
 * Interfaceのデフォルト実装(全部でOverrideするのはメンドイので)
 */
class BaseCrudService {
    constructor(parent) {
        this.parent = parent;
    }
    findAll(obj, asArray = true) {
        throw Error('Not implemented yet.');
    }
    find(obj) {
        throw Error('Not implemented yet.');
    }
    create(obj) {
        throw Error('Not implemented yet.');
    }
    update(obj) {
        throw Error('Not implemented yet.');
    }
    delete(obj) {
        throw Error('Not implemented yet.');
    }
    save2Excel(instances, outputFullPath, templateFullPath = '', sheetName = 'Sheet1', applyStyles) {
        const converters = {
        // RobotVersions: (value: any) => value[0].Count,
        };
        return excel_csv_read_write_1.json2excel(instances, outputFullPath, templateFullPath, sheetName, converters, applyStyles);
    }
    save2ExcelBlob(instances, sheetName = 'Sheet1', applyStyles) {
        const converters = {
        // RobotVersions: (value: any) => value[0].Count,
        };
        return excel_csv_read_write_1.json2excelBlob(instances, sheetName, converters, applyStyles);
    }
}
exports.BaseCrudService = BaseCrudService;
/**
 * OrchestratorのAPIのWrapperクラス
 * (Mainのクラスです)
 */
class OrchestratorApi {
    constructor(config) {
        this.config = config;
        this.isEnterprise = false;
        this.isCommunity = false;
        this.isRobot = false;
        this.isToken = false;
        this.accessToken = '';
        this.license = new LicenseCrudService_1.LicenseCrudService(this);
        this.robot = new RobotCrudService_1.RobotCrudService(this);
        this.user = new UserCrudService_1.UserCrudService(this);
        this.role = new RoleCrudService_1.RoleCrudService(this);
        this.machine = new MachineCrudService_1.MachineCrudService(this);
        this.release = new ReleaseCrudService_1.ReleaseCrudService(this);
        this.process = new ProcessCrudService_1.ProcessCrudService(this);
        this.library = new LibraryCrudService_1.LibraryCrudService(this);
        this.job = new JobCrudService_1.JobCrudService(this);
        this.schedule = new (class extends BaseCrudService {
            constructor(parent) {
                super(parent);
            }
            findAll(queries, asArray = true) {
                return utils_1.getArray(this.parent.config, this.parent.accessToken, '/odata/ProcessSchedules', queries, asArray);
            }
        })(this);
        this.folder = new FolderCrudService_1.FolderCrudService(this);
        this.tenant = new TenantCrudService_1.TenantCrudService(this);
        this.hostLicense = new HostLicenseCrudService_1.HostLicenseCrudService(this);
        this.environment = new EnvironmentCrudService_1.EnvironmentCrudService(this);
        this.queueDefinition = new QueueDefinitionCrudService_1.QueueDefinitionCrudService(this);
        this.queueItem = new QueueItemCrudService_1.QueueItemCrudService(this);
        this.queueOperation = new QueueCrudService_1.QueueCrudService(this);
        this.log = new LogCrudService_1.LogCrudService(this);
        this.auditLog = new AuditLogCrudService_1.AuditLogCrudService(this);
        this.setting = new SettingCrudService_1.SettingCrudService(this);
        this.asset = new AssetCrudService_1.AssetCrudService(this);
        this.util = new UtilService_1.UtilService(this);
        // タスク(Enterpriseには、ない)
        // フォルダー(OU)
        // Webhook
        this.getArray = (apiPath, queries, isOdata = true) => {
            return utils_1.getArray(this.config, this.accessToken, apiPath, queries, isOdata);
        };
        this.getData = (apiPath) => {
            return utils_1.getData(this.config, this.accessToken, apiPath);
        };
        this.postData = (apiPath, obj) => {
            return utils_1.postData(this.config, this.accessToken, apiPath, obj);
        };
        this.putData = (apiPath, obj) => {
            return utils_1.putData(this.config, this.accessToken, apiPath, obj);
        };
        this.deleteData = (apiPath) => {
            return utils_1.deleteData(this.config, this.accessToken, apiPath);
        };
        const { isEnterprise, isCommunity, isRobot, isToken } = exports.getConfigState(config);
        this.isEnterprise = isEnterprise;
        this.isCommunity = isCommunity;
        this.isRobot = isRobot;
        this.isToken = isToken;
        // 全部falseだったら、エラーなので例外で落とす。
        if (!(this.isEnterprise || this.isCommunity || this.isRobot)) {
            throw new Error('Invalid Configuration.');
        }
    }
    get organizationUnitId() {
        return this.config.organizationUnitIdFromAPI;
    }
    set organizationUnitId(organizationUnitId) {
        logger.info(`Folder Idを[${organizationUnitId}]に切り替えます`);
        this.config = Object.assign({}, this.config, { organizationUnitIdFromAPI: organizationUnitId });
    }
    /**
     * Orchestrator API の認証システムに対して、認証を実施しアクセストークンを取得する。
     */
    authenticate() {
        logger.debug(this.config.serverinfo.servername);
        // Enterprise版かCommunity版かで認証処理が異なるので、設定ファイルによって振り分ける。
        let promise;
        if (this.config.token && this.config.token.access_token !== '') {
            logger.info('Tokenモードとして処理開始');
            this.accessToken = this.config.token.access_token;
            // とりあえず、マシン一覧を取ることで、エラーが出たらトークンがつかえないと見なす。
            promise = new Promise((resolve, reject) => {
                utils_1.getArray(this.config, this.accessToken, '/odata/Machines')
                    .then(() => resolve())
                    .catch((error) => reject(error));
            });
        }
        else if (this.isRobot) {
            logger.info('Robotモードとして処理開始');
            logger.debug(this.config.robotInfo.machineKey);
            logger.debug(this.config.robotInfo.machineName);
            logger.debug(this.config.robotInfo.userName);
            const auth_options = utils_1.createRobotOption(this.config);
            promise = new Promise((resolve, reject) => {
                request_1.default.post(auth_options, (err, _response, body) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    const obj = body;
                    if (!obj.robotKey) {
                        reject(obj);
                        return;
                    }
                    const access_token = obj.robotKey;
                    this.accessToken = access_token;
                    logger.debug(access_token);
                    resolve(obj);
                });
            });
        }
        else if (this.isEnterprise) {
            logger.info('Enterprise版として処理開始');
            logger.debug(this.config.userinfo.tenancyName);
            logger.debug(this.config.userinfo.usernameOrEmailAddress);
            logger.debug(this.config.userinfo.password);
            const auth_options = utils_1.createEnterpriseOption(this.config);
            promise = new Promise((resolve, reject) => {
                request_1.default.post(auth_options, (err, _response, body) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    // const obj = JSON.parse(body)
                    const obj = body;
                    if (!obj.success) {
                        reject(obj);
                        return;
                    }
                    const access_token = obj.result;
                    this.accessToken = access_token;
                    logger.debug(access_token);
                    resolve(obj);
                });
            });
        }
        else {
            logger.info('Community版として処理開始');
            const auth_options = utils_1.createCommunityOption(this.config);
            promise = new Promise((resolve, reject) => {
                request_1.default.post(auth_options, (err, _response, body) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    // const obj = JSON.parse(body)
                    const obj = body;
                    if (!obj.access_token) {
                        reject(obj);
                        return;
                    }
                    logger.debug(obj);
                    const access_token = obj.access_token;
                    this.accessToken = access_token;
                    logger.debug(access_token);
                    resolve(obj);
                });
            });
        }
        return promise;
    }
}
exports.OrchestratorApi = OrchestratorApi;
/**
 * 与えられたConfigの状態
 */
exports.getConfigState = (config) => {
    let isEnterprise = false;
    let isCommunity = false;
    let isRobot = false;
    let isToken = false;
    if (config.token && config.token.access_token !== '') {
        isToken = true;
    }
    if (config.serverinfo) {
        // Enterpriseだったら、trueにする
        if (!config.serverinfo.tenant_logical_name) {
            // serverinfo.tenant_logical_nameプロパティがなければEnterprise
            isEnterprise = true;
        }
        else {
            //
        }
        isCommunity = !isEnterprise; // Enterpriseの逆にする。
        // Enterprise/Community判定は tenant_logical_name があるなしだけの判定なので、
        // tenant_logical_nameナシかつ robotInfoだけあれば、userinfoなくてもロボットモードで動くようにする
        if (config.robotInfo) {
            isRobot = true;
        }
        else {
            //
        }
    }
    return { isEnterprise, isCommunity, isRobot, isToken };
};
exports.default = OrchestratorApi;
const RobotCrudService_1 = require("./services/RobotCrudService");
const RoleCrudService_1 = require("./services/RoleCrudService");
const MachineCrudService_1 = require("./services/MachineCrudService");
const ReleaseCrudService_1 = require("./services/ReleaseCrudService");
const ProcessCrudService_1 = require("./services/ProcessCrudService");
const JobCrudService_1 = require("./services/JobCrudService");
const QueueDefinitionCrudService_1 = require("./services/QueueDefinitionCrudService");
const LogCrudService_1 = require("./services/LogCrudService");
const AuditLogCrudService_1 = require("./services/AuditLogCrudService");
const SettingCrudService_1 = require("./services/SettingCrudService");
const QueueCrudService_1 = require("./services/QueueCrudService");
const UserCrudService_1 = require("./services/UserCrudService");
const AssetCrudService_1 = require("./services/AssetCrudService");
const UtilService_1 = require("./services/UtilService");
const TenantCrudService_1 = require("./services/TenantCrudService");
const HostLicenseCrudService_1 = require("./services/HostLicenseCrudService");
const EnvironmentCrudService_1 = require("./services/EnvironmentCrudService");
const QueueItemCrudService_1 = require("./services/QueueItemCrudService");
const LibraryCrudService_1 = require("./services/LibraryCrudService");
const FolderCrudService_1 = require("./services/FolderCrudService");
const LicenseCrudService_1 = require("./services/LicenseCrudService");
// const getConfig = () => {
//   // 設定ファイルから読むパタン
//   return config
//   // Own Codingするパタン
//   // return {
//   //   userinfo: {
//   //     tenancyName: 'default',
//   //     usernameOrEmailAddress: 'aaa',
//   //     password: 'bbb',
//   //   },
//   //   serverinfo: {
//   //     servername: 'https://platform.uipath.com/',
//   //   },
//   // }
// }
// if (!module.parent) {
//   async function main() {
//     const api: IOrchestratorApi = new OrchestratorApi(getConfig())
//     try {
//       // まずは認証
//       await api.authenticate()
//       let instances: any[] = []
//       // instances = await api.asset.findAll({ $expand: 'RobotValues' })
//       instances = await api.asset.findAllEx()
//       console.table(instances)
//       // console.log(instances[0])
//       // api.asset.save2Excel(instances, 'assets.xlsx')
//       // await api.asset.upload('assets.xlsx')
//       await api.asset.uploadPerRobot('assets.xlsx', 'perRobot_assets.xlsx')
//       // instances = await api.queueItem.findAll()
//       // for (const instance of instances) {
//       //   console.log(instance)
//       // }
//       // console.log(instances.length)
//       // const queueItemId = instances[0].Id
//       // const result = await api.queueItem.find(queueItemId)
//       // console.log(result)
//       // let queueDef = await api.queueDefinition.findByName('QueueTest')
//       // // console.table(queueDef)
//       // queueDef = await api.queueDefinition.find(1)
//       // console.table(queueDef)
//     } catch (error) {
//       console.log(error)
//     }
//   }
//   main()
// }
//# sourceMappingURL=index.js.map