"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FolderCrudService = void 0;
const __1 = require("..");
const utils_1 = require("../utils");
const path_1 = __importDefault(require("path"));
class FolderCrudService extends __1.BaseCrudService {
    constructor(parent) {
        super(parent);
    }
    findAll(queries, asArray = true) {
        return utils_1.getArray(this.parent.config, this.parent.accessToken, '/odata/Folders', queries, asArray);
    }
    find(id) {
        return utils_1.getData(this.parent.config, this.parent.accessToken, `/odata/Folders(${id})`);
    }
    _findByName(name) {
        return utils_1.getArray(this.parent.config, this.parent.accessToken, '/odata/Folders', {
            $filter: `DisplayName eq '${name}'`,
        });
    }
    async findByDisplayName(name) {
        const defs = await this._findByName(name);
        return defs[0];
    }
    create(folder) {
        return utils_1.postData(this.parent.config, this.parent.accessToken, '/odata/Folders', folder);
    }
    update(folder) {
        return utils_1.putData(this.parent.config, this.parent.accessToken, `/odata/Folders(${folder.Id})`, folder);
    }
    delete(id) {
        return utils_1.deleteData(this.parent.config, this.parent.accessToken, `/odata/Folders(${id})`);
    }
    assignUsers(folderId, userIds) {
        return utils_1.postData(this.parent.config, this.parent.accessToken, '/odata/Folders/UiPath.Server.Configuration.OData.AssignUsers', {
            assignments: {
                UserIds: userIds,
                RolesPerFolder: [{ FolderId: folderId }],
            },
        });
    }
    removeUser(folderId, userId) {
        return utils_1.postData(this.parent.config, this.parent.accessToken, `/odata/Folders(${folderId})/UiPath.Server.Configuration.OData.RemoveUserFromFolder`, { userId: userId });
    }
    getUsers(folderId, queries, asArray = true) {
        return utils_1.getArray(this.parent.config, this.parent.accessToken, `/odata/Folders/UiPath.Server.Configuration.OData.GetUsersForFolder(key=${folderId},includeInherited=true)`, queries, asArray);
    }
    getFolders(userName) {
        return this.parent.getData(`/odata/Folders/UiPath.Server.Configuration.OData.GetAllRolesForUser(username='${userName}',skip=0,take=30)`);
    }
    removeFolders(folderIds, userDelete = false) {
        return cleanAll(this.parent, folderIds, userDelete);
    }
    async save2Excel(instances, outputFullPath, templateFullPath = path_1.default.join(__dirname, 'templates', 'templateFolders.xlsx'), sheetName = 'Sheet1', applyStyles) {
        // 全ユーザの情報(ただしTypeがRobotは覗く)を取得し、ユーザ毎に参照可能なフォルダを取得する。
        const allUsers = await this.parent.user.findAll();
        const userNames = allUsers
            .filter((user) => user.Type === 'User' || user.Type === 'DirectoryUser' || user.Type === 'DirectoryGroup')
            .map((user) => user.UserName);
        const folders = await this._getUserFolders(userNames);
        // console.table(folders)
        // その後Excelへ出力。
        const outputFullName = path_1.default.basename(outputFullPath);
        const templateFullName = path_1.default.basename(templateFullPath);
        const outputDir = path_1.default.dirname(outputFullPath);
        const templateDir = path_1.default.dirname(templateFullPath);
        const promises = Promise.all([
            super.save2Excel(instances, outputFullPath, templateFullPath, sheetName, applyStyles),
            super.save2Excel(folders, path_1.default.join(outputDir, `perUser_${outputFullName}`), path_1.default.join(templateDir, `perUser_${templateFullName}`), sheetName, applyStyles),
        ]);
        return new Promise((allResolve, reject) => {
            promises.then((results) => allResolve(results[0]));
        });
    }
    async _getUserFolders(userNames) {
        const allFolders = [];
        for (const userName of userNames) {
            const foldersInfo = await this.parent.folder.getFolders(userName);
            const folders = foldersInfo.PageItems.map((item) => item.Folder);
            const foldersWithUser = folders.map((folder) => Object.assign({}, folder, { UserName: userName }));
            allFolders.push(...foldersWithUser);
        }
        return allFolders;
    }
}
exports.FolderCrudService = FolderCrudService;
const cleanAll = async (api, folderIds, userDelete = false) => {
    for (const folderId of folderIds) {
        // このフォルダを閲覧出来るユーザを検索
        const users = await api.folder.getUsers(folderId); // adminも含まれる
        for (const user of users) {
            await api.folder.removeUser(folderId, user.Id);
            if (userDelete && user.UserEntity.UserName !== 'admin') {
                await api.user.delete(user.Id); //さっきつくったユーザも削除
            }
        }
        await api.folder.delete(folderId); // folderも削除
    }
};
//# sourceMappingURL=FolderCrudService.js.map