"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogCrudService = void 0;
const __1 = require("..");
const utils_1 = require("../utils");
const path_1 = __importDefault(require("path"));
const logger_1 = require("../logger");
const logger = logger_1.getLogger('main');
class LogCrudService extends __1.BaseCrudService {
    constructor(parent) {
        super(parent);
    }
    findAll(queries, asArray = true) {
        return utils_1.getArray(this.parent.config, this.parent.accessToken, '/odata/RobotLogs', queries, asArray);
    }
    async findByFilter(filters, obj, asArray = true) {
        const filterArray = await utils_1.createFilterStr(filters, this.parent);
        const filter = filterArray.join(' and ');
        if (filter === '') {
            return this.findAll(obj, asArray);
        }
        let condition = {};
        if (obj) {
            condition = obj;
            condition['$filter'] = filter;
        }
        else {
            condition = { $filter: filter };
        }
        return this.findAll(condition, asArray);
    }
    async findStartEndLogs(filters, obj) {
        const results = await this.findByFilter(filters, obj);
        return Promise.all(results
            .filter((data) => {
            const message = data.Message;
            const rawMessageObj = JSON.parse(data.RawMessage);
            if (rawMessageObj.hasOwnProperty('totalExecutionTimeInSeconds')) {
                return true;
            }
            else if (message.match(/の実行を開始しました/) || message.match(/execution started/)) {
                return true;
            }
            return false;
        })
            .map(async (data) => {
            let machineName = '';
            try {
                const machine = await this.parent.machine.find(data.MachineId);
                machineName = machine.Name;
            }
            catch (error) {
                logger.error(`StatusCode: ${error.statusCode}`);
                logger.error(error.body);
                if (error instanceof utils_1.NetworkAccessError) {
                    if (error.statusCode === 404) {
                        logger.error(`MachinId: ${data.MachineId}`);
                        // 404の場合は処理を継続
                    }
                    else {
                        throw error;
                    }
                }
            }
            delete data.Level;
            delete data.MachineId;
            const rawMessageObj = JSON.parse(data.RawMessage);
            if (rawMessageObj.hasOwnProperty('totalExecutionTimeInSeconds')) {
                return Object.assign({}, data, {
                    MachineName: machineName,
                    LogType: 'end',
                    TotalExecutionTimeInSeconds: rawMessageObj.totalExecutionTimeInSeconds,
                });
            }
            else {
                return Object.assign({}, data, {
                    MachineName: machineName,
                    LogType: 'start',
                    TotalExecutionTimeInSeconds: 0,
                });
            }
        }));
    }
    save2Excel(logs, outputFullPath, templateFullPath = path_1.default.join(__dirname, 'templates', 'templateLogs.xlsx'), // テンプレファイルは、指定されたファイルか、このソースがあるディレクトリ上のtemplateLogs.xlsxを使う
    sheetName = 'Sheet1', applyStyles) {
        const applyStyles_ = applyStyles
            ? applyStyles
            : (logs_, workbook, sheetName_) => {
                const sheet = workbook.sheet(sheetName_);
                const rowCount = logs_.length;
                // A列に、J列にあるUTCデータから JST変換を行う関数を入れている。
                // I列は、なぜかゼロがNULL値になっているので、0を入れる処理を入れている。
                for (let i = 0; i < rowCount; i++) {
                    const rowIndex = i + 2;
                    sheet
                        .cell(`A${rowIndex}`)
                        .formula(`=DATEVALUE(MIDB(J${rowIndex},1,10))+TIMEVALUE(MIDB(J${rowIndex},12,8))+TIME(9,0,0)`);
                    if (logs_[i].TotalExecutionTimeInSeconds === 0) {
                        sheet.cell(`I${rowIndex}`).value(0);
                    }
                }
                // JSTの時刻を入れている箇所に、日付フォーマットを適用
                sheet.range(`A2:A${rowCount + 1}`).style('numberFormat', 'yyyy/mm/dd hh:mm:ss;@');
            };
        return super.save2Excel(logs, outputFullPath, templateFullPath, sheetName, applyStyles_);
    }
}
exports.LogCrudService = LogCrudService;
//# sourceMappingURL=LogCrudService.js.map