"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAuditFilterStr = exports.createFilterStr = exports.createRobotOption = exports.createCommunityOption = exports.createEnterpriseOption = exports.deleteData = exports.putData = exports.downloadData = exports.uploadData = exports.postData = exports.getArray = exports.getData = exports.NetworkAccessError = void 0;
const request_1 = __importDefault(require("request"));
const logger_1 = require("./logger");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const logger = logger_1.getLogger('main');
const httpLogger = logger_1.getLogger('httpLogger');
class BaseError extends Error {
    constructor(e) {
        super(e);
        this.name = new.target.name;
        // 下記の行はTypeScriptの出力ターゲットがES2015より古い場合(ES3, ES5)のみ必要
        Object.setPrototypeOf(this, new.target.prototype);
    }
}
class NetworkAccessError extends BaseError {
    constructor(statusCode, body, e) {
        super(e);
        this.statusCode = statusCode;
        this.body = body;
    }
}
exports.NetworkAccessError = NetworkAccessError;
exports.getData = (config, accessToken, apiPath) => {
    const option = createOption(config, accessToken, apiPath);
    return createStrPromise(Object.assign(option, { method: 'GET' }));
};
exports.getArray = (config, accessToken, apiPath, queries, isOdata = true) => {
    let optionTmp = createOption(config, accessToken, apiPath);
    optionTmp = Object.assign(optionTmp, { method: 'GET' });
    let option = optionTmp;
    if (queries) {
        option = Object.assign(optionTmp, { qs: queries });
    }
    return createArrayPromise(option, isOdata);
};
exports.postData = (config, accessToken, apiPath, obj) => {
    const option = createOption(config, accessToken, apiPath);
    return createJSONPromise(Object.assign(option, {
        method: 'POST',
        json: obj,
    }));
};
exports.uploadData = async (config, accessToken, apiPath, fullPath, isOdata = true) => {
    const option = createOption(config, accessToken, apiPath, 'multipart/form-data');
    const p = new Promise((resolve, reject) => {
        fs_1.default.readFile(fullPath, (err, data_) => {
            if (err) {
                reject(err);
                return;
            }
            resolve(data_);
            return;
        });
    });
    const data = await p;
    return createArrayPromise(Object.assign({}, option, {
        method: 'POST',
        multipart: [
            {
                'Content-Disposition': `form-data; name="uploads[]"; filename="${path_1.default.basename(fullPath)}"`,
                'Content-Type': 'application/octet-stream',
                'Content-Length': data.length,
                body: data,
            },
        ],
    }), isOdata);
};
exports.downloadData = (config, accessToken, apiPath, id, version) => {
    const option = createOption(config, accessToken, apiPath);
    option.headers['Accept'] = 'application/octet-stream';
    return createDownloadPromise(Object.assign({}, option, {
        method: 'GET',
        encoding: null,
    }), id, version);
};
exports.putData = (config, accessToken, apiPath, obj) => {
    const option = createOption(config, accessToken, apiPath);
    return createJSONPromise(Object.assign(option, {
        method: 'PUT',
        json: obj,
    }));
};
exports.deleteData = (config, accessToken, apiPath) => {
    const option = createOption(config, accessToken, apiPath);
    return createJSONPromise(Object.assign(option, {
        method: 'DELETE',
    }));
};
/**
 * 「APIのReturn値のvalueが配列であると見なしてvalueを配列で返すPromise」を返す
 * @param option
 * @param isOdata  stat系APIは、OData形式ではなく、どうもただの配列で返ってくるので、valueプロパティを返すかそのまま返すかの判定フラグに使う
 */
const createArrayPromise = (option, isOdata) => {
    // promiseを返す処理は毎回おなじ。Request処理して、コールバックで値を設定するPromiseを作って返すところを共通化
    const promise = new Promise((resolve, reject) => {
        request_1.default(option, function (err, response, body) {
            if (err) {
                reject(err);
                return;
            }
            httpLogger.debug({ objects: option });
            logger.info(`method: ${option.method}, statuCode: ${response.statusCode}`);
            if (response.statusCode >= 400) {
                logger.error(body);
                reject(new NetworkAccessError(response.statusCode, body));
            }
            logger.debug(body);
            const obj = JSON.parse(body);
            if (isOdata) {
                resolve(obj.value); // valueプロパティが配列である場合。
            }
            else {
                resolve(obj); // objがそのまま配列であるばあい。
            }
        });
    });
    return promise;
};
/**
 * 「APIのReturn値をそのまま返すPromise」を返す
 * @param option
 */
const createStrPromise = (option) => {
    const promise = new Promise((resolve, reject) => {
        request_1.default(option, function (err, response, body) {
            if (err) {
                reject(err);
                return;
            }
            httpLogger.debug({ objects: option });
            logger.info(`method: ${option.method}, statuCode: ${response.statusCode}`);
            if (response.statusCode >= 400) {
                logger.error(body);
                reject(new NetworkAccessError(response.statusCode, body));
            }
            if (body === null || body === '') {
                resolve();
                return;
            }
            const obj = JSON.parse(body);
            // エラーがあった場合
            if (obj.errorCode || obj.errorCode === 0) {
                //errorCodeが数値のゼロの時があった
                reject(obj);
                return;
            }
            resolve(obj);
            return;
        });
    });
    return promise;
};
/**
 * 「APIのReturn値をそのまま返すPromise」を返す
 * @param option
 */
const createJSONPromise = (option) => {
    const promise = new Promise((resolve, reject) => {
        request_1.default(option, function (err, response, body) {
            if (err) {
                reject(err);
                return;
            }
            httpLogger.debug({ objects: option });
            logger.info(`method: ${option.method}, statuCode: ${response.statusCode}`);
            if (response.statusCode >= 400) {
                logger.error(body);
                reject(new NetworkAccessError(response.statusCode, body));
            }
            // PUTのばあい、StatusCodeが200で、Bodyが空のためundefinedになったが正常終了させる。
            if (response.statusCode === 200 && !body) {
                resolve();
                return;
            }
            // POSTでなげて204(当然Body部が空文字)が返ってきたとき、bodyがundefinedになった   ←この件の対応をいれた if(body...)
            // DELETEでなげて204(戻り電文はPOSTと同じ空) が返ったときは、bodyは''になった
            // つまり同じ空電文でも、POST/DELETEで、requestライブラリのbodyの値は異なるということだ
            if (body && (body.errorCode || body.errorCode === 0)) {
                // エラーがあった場合
                //errorCodeが数値のゼロの時があった
                reject(body);
                return;
            }
            resolve(body);
            return;
        });
    });
    return promise;
};
const createDownloadPromise = (option, id, version) => {
    return new Promise((resolve, reject) => {
        request_1.default(option, (err, response, body) => {
            if (err) {
                reject(err);
                return;
            }
            httpLogger.debug({ objects: option });
            logger.info(`method: ${option.method}, statuCode: ${response.statusCode}`);
            if (response.statusCode >= 400) {
                logger.error(body);
                reject(new NetworkAccessError(response.statusCode, body));
            }
            if (response.statusCode === 200) {
                fs_1.default.writeFileSync(`${id}.${version}.nupkg`, body, 'binary');
                resolve();
            }
            else {
                reject(err);
            }
        });
    });
};
const createOption = (config, accessToken, apiPath, contentType = 'application/json') => {
    const option = {
        // uri: url.resolve(servername, apiPath),
        uri: config.serverinfo.servername + apiPath,
        headers: headers(config, accessToken, contentType),
    };
    return addAdditionalOption(config, option);
};
exports.createEnterpriseOption = (config) => {
    const option_tmp = {
        uri: config.serverinfo.servername + '/api/Account/Authenticate',
        headers: {
            // 'Content-Type': 'application/x-www-form-urlencoded',
            'Content-Type': 'application/json',
            Accept: 'application/json',
        },
        // form: config.userinfo,
        json: config.userinfo,
    };
    return addAdditionalOption(config, option_tmp);
};
exports.createCommunityOption = (config) => {
    const form = Object.assign(config.serverinfo, {
        grant_type: 'refresh_token',
    });
    const option_tmp = {
        uri: 'https://account.uipath.com/oauth/token',
        headers: {
            'Content-Type': 'application/json',
            Accept: 'application/json',
        },
        json: form,
    };
    return addAdditionalOption(config, option_tmp);
};
exports.createRobotOption = (config) => {
    const option_tmp = {
        uri: config.serverinfo.servername + '/api/robotsservice/BeginSession',
        headers: {
            'Content-Type': 'application/json; charset=utf-8',
            'X-ROBOT-LICENSE': config.robotInfo.machineKey,
            'X-ROBOT-MACHINE-ENCODED': Buffer.from(config.robotInfo.machineName).toString('base64'),
            Accept: 'application/json',
        },
        json: { UserName: config.robotInfo.userName },
    };
    return addAdditionalOption(config, option_tmp);
};
const addAdditionalOption = (config, option) => {
    if ('strictSSL' in config.serverinfo) {
        Object.assign(option, { strictSSL: config.serverinfo.strictSSL });
        // process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';
    }
    if (config.proxy) {
        // プロパティ proxy があって
        if (config.proxy.useProxy) {
            // useProxy がtrue ならプロキシを設定する
            return Object.assign({}, option, {
                proxy: config.proxy.url,
                strictSSL: false,
            });
        }
    }
    return option;
};
const headers = (config, accessToken, contentType) => {
    const tenant_logical_name = config.serverinfo.tenant_logical_name;
    let ret = {};
    if (config.robotInfo) {
        ret = {
            Authorization: 'UiRobot ' + accessToken,
            'content-type': contentType,
        };
        if (config.robotInfo.organizationUnit) {
            ret = Object.assign({}, ret, { 'X-UIPATH-OrganizationUnitId': config.robotInfo.organizationUnit });
        }
    }
    else {
        ret = {
            Authorization: 'Bearer ' + accessToken,
            'content-type': contentType,
        };
        if ('userinfo' in config) {
            if (config.userinfo.organizationUnit) {
                ret = Object.assign({}, ret, { 'X-UIPATH-OrganizationUnitId': config.userinfo.organizationUnit });
            }
        }
    }
    // しかし、organizationUnitIdFromAPI があればそれ優先
    if ('organizationUnitIdFromAPI' in config) {
        ret = Object.assign({}, ret, { 'X-UIPATH-OrganizationUnitId': config.organizationUnitIdFromAPI });
    }
    if (tenant_logical_name) {
        return Object.assign(ret, {
            'X-UiPath-TenantName': tenant_logical_name,
        });
    }
    return ret;
};
exports.createFilterStr = async (filters, api) => {
    const ret = [];
    if (filters.from) {
        const fromUTC = filters.from.toISOString();
        logger.debug(`from: ${fromUTC}`);
        ret.push(`TimeStamp ge ${fromUTC}`);
    }
    if (filters.to) {
        const toUTC = filters.to.toISOString();
        logger.debug(`  to: ${toUTC}`);
        ret.push(`TimeStamp lt ${toUTC}`);
    }
    if (filters.robotName) {
        ret.push(`RobotName eq '${filters.robotName}'`);
    }
    if (filters.processName) {
        ret.push(`ProcessName eq '${filters.processName}'`);
    }
    if (filters.windowsIdentity) {
        ret.push(`WindowsIdentity eq '${filters.windowsIdentity}'`);
    }
    if (filters.level) {
        ret.push(`Level eq '${filters.level}'`);
    }
    if (filters.machineName) {
        const machine = await api.machine.findByMachineName(filters.machineName);
        ret.push(`MachineId eq ${machine.Id}`);
    }
    return ret;
};
exports.createAuditFilterStr = async (filters, api) => {
    const ret = [];
    if (filters.from) {
        const fromUTC = filters.from.toISOString();
        logger.debug(`from: ${fromUTC}`);
        ret.push(`ExecutionTime ge ${fromUTC}`);
        // ret.push(`ExecutionTime lt ${fromUTC}`)
    }
    if (filters.to) {
        const toUTC = filters.to.toISOString();
        logger.debug(`  to: ${toUTC}`);
        ret.push(`ExecutionTime lt ${toUTC}`);
    }
    if (filters.action) {
        ret.push(`Action eq '${filters.action}'`);
    }
    if (filters.userName) {
        ret.push(`UserName eq '${filters.userName}'`);
    }
    if (filters.component) {
        ret.push(`Component eq '${filters.component}'`);
    }
    if (filters.methodName) {
        ret.push(`MethodName eq '${filters.methodName}'`);
    }
    return ret;
};
if (!module.parent) {
    //   main()
}
//# sourceMappingURL=utils.js.map